@@ -1,7 +1,7 @@
 /*
  * icon.cc
  *
- * Copyright (C) 1995-2000 Kenichi Kourai
+ * Copyright (C) 1995-2001 Kenichi Kourai
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -413,9 +413,11 @@ void Icon::Button1Press(Time clickTime, const Point& ptRoot)
    */
   if (IsDoubleClick(iconClickTime, clickTime, event.ptPrevRoot, ptRoot)) {
     if (isBuiltin)
-      rootQvwm->ExecFunction(fn);
-    else
+      QvFunction::execFunction(fn);
+    else {
+      PlaySound(OpenSound);
       ExecCommand(exec);
+    }
     return;
   }
   else
@@ -430,6 +432,9 @@ void Icon::Button1Motion(const Point& ptRoot)
   Point ptOld, ptNew, ptCur;
   Bool pointer = False;
 
+  if (DisableDesktopChange)
+    return;
+
   ptCur = Point(rc.x, rc.y);
   ptOld = ptRoot;
 
@@ -681,9 +686,11 @@ void IconMenu::ExecFunction(FuncNumber fn, int i)
   case Q_EXEC_ICON:
     if (icon) {
       if (icon->IsBuiltin())
-	rootQvwm->ExecFunction(icon->GetFunc());
-      else
+	QvFunction::execFunction(icon->GetFunc());
+      else {
+	PlaySound(OpenSound);
 	ExecCommand(icon->GetExec());
+      }
     }
     break;
 
