@@ -1,7 +1,7 @@
 /*
  * system_dialog.cc
  *
- * Copyright (C) 1995-2000 Kenichi Kourai
+ * Copyright (C) 1995-2001 Kenichi Kourai
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -232,6 +232,8 @@ void SystemDialog::MapDialog()
 
 void SystemDialog::UnmapDialog()
 {
+  Dialog::UnmapDialog();
+
   XUnmapWindow(display, parent);
 }
 
@@ -254,7 +256,7 @@ void SystemDialog::DrawDialog()
   xp[2].x = 0;
   xp[2].y = height - 2;
 
-  XSetForeground(display, ::gc, lightGray.pixel);
+  XSetForeground(display, ::gc, gray.pixel);
   XDrawLines(display, parent, ::gc, xp, 3, CoordModeOrigin);
   
   xp[0].x = width - 1;
@@ -264,7 +266,7 @@ void SystemDialog::DrawDialog()
   xp[2].x = 0;
   xp[2].y = height - 1;
 
-  XSetForeground(display, ::gc, black.pixel);
+  XSetForeground(display, ::gc, darkGrey.pixel);
   XDrawLines(display, parent, ::gc, xp, 3, CoordModeOrigin);
 
   xp[0].x = width - 3;
