@@ -1,7 +1,7 @@
 /*
  * window.cc
  *
- * Copyright (C) 1995-2000 Kenichi Kourai
+ * Copyright (C) 1995-2001 Kenichi Kourai
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -321,10 +321,7 @@ void Qvwm::RaiseWindow(Bool soon)
 */
 
     if (this != activeQvwm)
-      for (int i = 1; i < 4; i++)
-	XUngrabButton(display, i, 0, wOrig);
-
-    activeQvwm = this;
+      activeQvwm = this;
   }
 }
 
